<?php

require_once 'Horde/DataTree.php';
require_once 'Horde/Group.php';

/**
 * Abstract Thor_Project Class.
 * Every kind of Project in Thor is an extension of that class which defines common
 * fields to every project as well as some methods.
 * Each kind of Project must have a list of available and required Items
 *
 * $Horde: thor/lib/Project.php,v 1.41 2004/12/20 19:14:53 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @package Thor
 */
class Thor_Projects {

    /**
     * DataTree instance holding all the projects.
     */
    var $_projects;

    /**
     * Constructor
     */
    function Thor_Projects()
    {
        global $conf;
        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde DataTree backend to use Thor."), __FILE__, __LINE__);
        }
        $driver = $conf['datatree']['driver'];
        $params = array_merge(Horde::getDriverConfig('datatree', $driver), array('group' => 'thor.projects'));

        /* Connect to the backend for tracking forums. */
        $this->_projects = &DataTree::singleton($driver, $params);
    }

    function &singleton()
    {
        static $projectsOb;

        if (!isset($projectsOb)) {
            $projectsOb = new Thor_Projects();
        }

        return $projectsOb;
    }

    /**
     * Projects factory
     * This function does also set the creation timestamp and the creator id
     * TODO: Check the creation policy!!
     *
     * @param string $project_path
     * @param string $kind The kind of Project we want to create
     * @return mixed PEAR_Error if this kind of Project is unknown. The Project if all goes ok.
     */
    function &newProject($project_path, $kind)
    {
        require_once dirname(__FILE__) . '/Project/' . $kind . '.php';
        $class = 'DataTreeObject_' . $kind;
        if (class_exists($class)) {
            $project = &new $class($project_path);
            return $project;
        } else {
            return PEAR::raiseError(sprintf(_("'%s' not found."), $class));
        }
    }

    /**
     * Get a Project given its ID
     * @param string $project_id The Project identifier
     * @return mixed DataTreeObject_Project instance if all went ok. PEAR_Error otherwise.
     */
    function &getProject($project_id)
    {
        $project = &$this->_projects->getObjectById($project_id, null);
        return $project;
    }

    /**
     * Get a Project ID given its name
     */
    function getProjectId($project_name)
    {
        return $this->_projects->getId($project_name);
    }

    function getProjectParent($project)
    {
        return $this->_projects->getParent($project);
    }

    /**
     * This should be modified to use the method removeById which is
     * not yet implemented in the DataTree driver.
     *
     * @param integer $project_id The project identifier
     * @return mixed PEAR_Error if there was any problem. Id of the project otherwise
     */
    function deleteProject($project_id)
    {
        $project = &$this->getProject($project_id);
        $project->_destroy();
        return $this->_projects->remove($project->getName());
    }

    /**
     * Function that updates the Last Modified Date of the Project
     * It does also update all the parents of the project
     *
     * @param integer $projectID The project identifier
     * @return mixed PEAR_Error if something went wrong. True if all went ok.
     */
    function updateModifiedDate($projectID)
    {
        if ($projectID == DATATREE_ROOT) {
            return true;
        }
        //Update the projects
        $toUpdate = $this->_projects->getParentList($projectID, array($projectID => ''));
        $idsList = array_keys($toUpdate);
        $now = time();
        foreach($idsList as $id) {
            $project = &$this->getProject($id);
            $project->set('project_last_modified', $now);
            $result = $this->_projects->updateData($project);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }
        return true;
    }

    /**
     * Function that stores the Project data on the backend
     * It takes care of adding a new entry if necessary
     * Updates Last Changed field
     * TODO: Check for Path changes...
     * @param mixed $info The Project data array or a DataTreeObject_Project instance
     * @return mixed The project id if all went ok. PEAR_Error if there were any problem
     */
    function saveProject(&$info)
    {
        if (is_a($info, 'DataTreeObject_Project')) {
            //Existing project
            $info->set('project_last_modified', time());
            $result = $this->_projects->updateData($info);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            $parentID = $this->getProjectParent($info);
            $this->updateModifiedDate($parentID);
            return $info->get('project_id');
        }

        if (empty($info['project_id'])) {
            $project_name = $info['project_name'];
            $project_kind = $info['project_kind'];
            $project_parent_id = $info['project_parent_id'];
            if ($project_parent_id != DATATREE_ROOT) {
                $project_path = $this->_projects->getName($project_parent_id);
                $project_path .= ':' . DataTree::encodeName($project_name);
            } else {
                $project_path = DataTree::encodeName($project_name);
            }
            $project = &$this->newProject($project_path, $project_kind);
        } else {
            $project = &$this->getProject($info['project_id']);
        }
        $project->fromArray($info);
        if (empty($info['project_id'])) {
            // Its a new Project
            $this->updateModifiedDate($project_parent_id);
            return $this->_projects->add($project);
        } else {
            //Existing project
            $project->set('project_last_modified', time());
            $result = $this->_projects->updateData($project);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            $parentID = $this->getProjectParent($project);
            $this->updateModifiedDate($parentID);
            return $info['project_id'];
        }
    }

    /**
     * Retrieves all the projects a user is working on
     * It can also retrieve the current requests
     * @param string $userID The user name
     * @param integer $status The status of the projects to list
     * @param integer $role Role the user should have on that project. Default USERS_TEAM
     * @return mixed (name => Project_instance) or (Id => Name) if getObjects = false
     */
    function &listUserProjects($userID, $status, $role = USERS_TEAM, $getObjects = false)
    {
        $result_set = array();
        $projectList = &$this->getProjects($status);
        if (is_a($projectList, 'PEAR_Error')) {
            return $result_set;
        }

        /* Filter the projects list: only those where the user is working on */
        foreach ($projectList as $projectID => $projectName) {
            $theProject = &$this->getProject($projectID);
            if ($theProject->isUserOfKind($userID, $role)) {
                if ($getObjects) {
                    $result_set[$projectName] = $theProject;
                } else {
                    $result_set[$projectID] = $projectName;
                }
            }
        }
        return $result_set;
    }

    /**
     * Function that retrieves the list of Projects
     * @param integer $status The status of the projects to list
     * @param string $startleaf The node from which to start getting the list
     * @param integer $maxdepth The depth of the tree
     * @param integer $sortby Sorting parameter.
     * @param integer $sortdir Sorting parameter.
     * @param boolean $no_root True -> remove the root project from the result
     * @param boolean $getObjects True: get also the Projects instances
     * @return mixed All ok hash id => name or name => Project_instance if $getObjects is true
     */
    function &getProjects($status = THOR_STATUS_ANY, $startleaf = DATATREE_ROOT, $maxdepth = -1, $no_root = false, $getObjects = false)
    {
        if ($status != THOR_STATUS_ANY) {
            $criteria = array('AND' => array(
                array('field' => 'name', 'op'  => '=', 'test'  => 'project'),
                array('field' => 'key', 'op' => '=', 'test'  => 'status'),
                array('field' => 'value', 'op' => '&', 'test'  => $status)));

            $projectsList = $this->getProjectsByCriteria($criteria);
        } else {
            $projectsList = $this->_projects->get(DATATREE_FORMAT_FLAT, $startleaf, true,  _("-- Top Level --"), $maxdepth, true);
        }

        if (is_a($projectsList, 'PEAR_Error')) {
            Horde::logMessage($projectsList, __FILE__, __LINE__, PEAR_LOG_ERR);
            $projectsList = array();
            return $projectsList;
        }

        if ($no_root) {
            $projectID = $this->getProjectId($startleaf);
            unset($projectsList[$projectID]);
        }

        if ($getObjects) {
            $projectsList = &$this->_projects->getObjects(array_keys($projectsList), null);
        }

        return $projectsList;
    }

    /**
     * Function that retrieves the list of Projects based on the given criteria
     * @param array $criteria               The array of criteria. Example:
     *                                      $criteria['OR'] = array(
     *                                          array('AND' => array(
     *                                              array('field' => 'name',
     *                                                    'op'    => '=',
     *                                                    'test'  => 'foo'),
     *                                              array('field' => 'key',
     *                                                    'op'    => '=',
     *                                                    'test'  => 'abc'))),
     *                                          array('AND' => array(
     *                                              array('field' => 'name',
     *                                                    'op'    => '=',
     *                                                    'test'  => 'bar'),
     *                                              array('field' => 'key',
     *                                                    'op'    => '=',
     *                                                    'test'  => 'xyz'))));
     *                                      This would fetch all object ids
     *                                      where attribute name is "foo" AND
     *                                      key is "abc", OR "bar" AND "xyz".
     * @param string $startleaf The node from which to start getting the list
     *                                      from.
     * @param optional bool $allLevels      Return all levels, or just the
     *                                      direct children of $parent?
     *                                      Defaults to all levels.
     * @param boolean $getObjects True: get also the Projects instances
     * @return mixed All ok hash id => name or name => Project_instance if $getObjects is true
     */
    function &getProjectsByCriteria($criteria, $startleaf = DATATREE_ROOT, $allLevels = true, $getObjects = false)
    {
        $projectsList = $this->_projects->getByAttributes($criteria, $startleaf, $allLevels, true);

        if (is_a($projectsList, 'PEAR_Error')) {
            Horde::logMessage($projectsList, __FILE__, __LINE__, PEAR_LOG_ERR);
            $projectsList = array();
            return $projectsList;
        }

        if ($getObjects) {
            $projectsList = &$this->_projects->getObjects(array_keys($projectsList), null);
        } else {
            // Put the full name
            $Ids = array_keys($projectsList);
            foreach ($Ids as $projectId) {
                $projectsList[$projectId] = $this->_projects->getName($projectId);
            }
        }
        return $projectsList;
    }
}

/**
 * DataTreeObject_Project Class.
 * Every Project in Thor is an extension of that class which defines common
 * fields as well as methods all should implement
 *
 * $Horde: thor/lib/Project.php,v 1.41 2004/12/20 19:14:53 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @package Thor
 */
class DataTreeObject_Project extends DataTreeObject {

    /**
     * Variable required for Dynamic Binding of Projects
     */
    var $_objectType = 'thor/Project';

     // Caches the groups' names. Set on instance creation
    var $_top_group_name;
    var $_default_groups;

    function DataTreeObject_Project($name)
    {
        parent::DataTreeObject($name);
        $this->_top_group_name = DataTree::getShortName($name);
        $this->_default_groups = array (
            USERS_DEVS => $this->_buildGroupName(USERS_DEVS),
            USERS_ADMINS => $this->_buildGroupName(USERS_ADMINS),
            USERS_ACCEPTED => $this->_buildGroupName(USERS_ACCEPTED),
            USERS_WANTING => $this->_buildGroupName(USERS_WANTING),
            USERS_REFUSED => $this->_buildGroupName(USERS_REFUSED),
            USERS_DISCHARGED => $this->_buildGroupName(USERS_DISCHARGED));
        $this->set('project_coordinator_id', Auth::getAuth());
        $this->set('project_creation_date', time());
        $this->set('project_status', THOR_STATUS_REQUEST);
    }

    function _buildGroupName($role)
    {
        switch($role) {
        case USERS_ANY:
            return $this->_top_group_name;
            break;

        case USERS_ADMINS:
            return $this->_top_group_name . ':' . DataTree::encodeName('Administrators');
            break;

        case USERS_DEVS:
            return $this->_top_group_name . ':' . DataTree::encodeName('Developers');
            break;

        case USERS_WANTING:
            return $this->_top_group_name . ':' . DataTree::encodeName('Wanting');
            break;

        case USERS_ACCEPTED:
            return $this->_top_group_name . ':' . DataTree::encodeName('Accepted');
            break;

        case USERS_REFUSED:
            return $this->_top_group_name . ':' . DataTree::encodeName('Refused');
            break;

        case USERS_DISCHARGED:
            return $this->_top_group_name . ':' . DataTree::encodeName('Discharged');
            break;

        default:
            return false;
            break;
        }
    }

    function _toAttributes()
    {
        $attributes[] = array('name'  => 'DataTree',
                              'key'   => 'objectType',
                              'value' => $this->_objectType);
        $attributes[] = array('name'  => 'project',
                              'key'   => 'coordinator_id',
                              'value' => $this->data['project_coordinator_id']);
        $attributes[] = array('name'  => 'project',
                              'key'   => 'creation_date',
                              'value' => $this->data['project_creation_date']);
        $attributes[] = array('name'  => 'project',
                              'key'   => 'last_modified',
                              'value' => $this->data['project_last_modified']);
        $attributes[] = array('name'  => 'project',
                              'key'   => 'status',
                              'value' => $this->data['project_status']);
        $attributes[] = array('name'  => 'project',
                              'key'   => 'description',
                              'value' => $this->data['project_description']);
        if (!empty($this->data['project_write_protected'])) {
             $attributes[] = array('name'  => 'project',
                                  'key'   => 'write_protected',
                                  'value' => 1);
        }

        return $attributes;
    }

    function _fromAttributes($attributes)
    {
        /* Initialise the data array. */
        $this->data = array();

        foreach ($attributes as $attr) {
            $this->data[$attr['name'] . '_' . $attr['key']] = $attr['value'];
        }
    }

    function toArray()
    {
        global $thor;

        $array['project_id'] = $thor->_projectsContainer->getProjectId($this->getName());
        $array['project_parent_id'] = $thor->_projectsContainer->getProjectParent($this);
        $array['project_name'] = $this->getShortName();
        $array['project_coordinator_id'] = $this->data['project_coordinator_id'];
        $array['project_creation_date'] = $this->data['project_creation_date'];
        $array['project_last_modified'] = $this->data['project_last_modified'];
        $array['project_status'] = $this->data['project_status'];
        $array['project_description'] =  $this->data['project_description'];
        $array['project_kind'] =  substr(strrchr($this->_objectType, '/'), 1);
        $array['project_write_protected'] = empty($this->data['project_write_protected']) ? 0 : 1;
        return $array;
    }

    function fromArray(&$info)
    {
        $this->data['project_last_modified'] = time();
        $this->data['project_description'] = $info['project_description'];
    }

    /**
    * Get the project's users that match the criteria
    * TODO: Sort users
    * @param constant integer $criteria The kind(s) of users to list
    * @return mixed Array of user names or False if there was a problem.
    */
    function listUsers($criteria = USERS_ANY, $sortby, $sortdir)
    {
        $groups = &Group::singleton();

        if ($criteria == USERS_ANY) {
            $group_name = $this->_top_group_name;
            return $groups->listAllUsers($group_name);
        } elseif ($criteria == USERS_TEAM) {
            $group_name = $this->_default_groups[USERS_ADMINS];
            $admins = $groups->listAllUsers($group_name);
            $group_name = $this->_default_groups[USERS_DEVS];
            $devels = $groups->listAllUsers($group_name);
            return array_merge($admins,$devels);
        } else {
            $group_name = $this->_default_groups[$criteria];
            return $groups->listUsers($group_name);
        }
    }

    /**
      * This function returns true if the user has the role provided for the
      * given project.
      *
      * @param string $userID Name of the user we are checking
      * @param constant $role The role for the user in that project
      *
      * @return boolean True if the user has that role on the given project
     */
    function isUserOfKind($userID, $role)
    {
        $groups = &Group::singleton();

        if ( (empty($userID)) || (empty($role)) ) {
            return false;
        }

        switch ($role) {
        case USERS_ANY:
            $result = $groups->userIsInGroup($userID, $this->_top_group_name, false);
            break;

        case USERS_TEAM:
            $devs_name = $this->_default_groups[USERS_DEVS];
            $res = $groups->userIsInGroup($userID, $devs_name, false);
            if (is_a($res, 'PEAR_Error')) {
                return false;
            } elseif (!$res) {
                $admins_name = $this->_default_groups[USERS_ADMINS];
                $res = $groups->userIsInGroup($userID, $admins_name, false);
                if (is_a($res, 'PEAR_Error')) {
                    return false;
                }
            }
            return $res;
            break;

        case USERS_COORDINATOR:
            return ($this->data['project_coordinator_id'] == $userID);
            break;

        default:
            $group_name = $this->_default_groups[$role];
            $result = $groups->userIsInGroup($userID, $group_name, false);
            break;
        }

         return $result;
    }

    /**
     * This function changes a user's group from the old group to the new one
     * This is performed on the current project's scope
     *
     * @param string $userID                The user id.
     * @param string $oldGroupName     The name of group the user belongs to atm
     * @param string $newGroupName    Name of the new group the user will belong to
     * @return boolean True if the user could be moved to the new group. False otherwise.
     */
    function changeUserGroup($userID, $oldGroupName, $newGroupName)
    {
        $groups = &Group::singleton();

        $parentsOld = $groups->getGroupParents($oldGroupName);
        $parentsNew = $groups->getGroupParents($newGroupName);

        // Check we don't do anything out of the project's scope and the user was there already
        if ( isset($parentsOld[$this->_top_group_name]) &&
              isset($parentsNew[$this->_top_group_name]) &&
              $groups->userIsInGroup($userID, $oldGroupName, false)) {

            $old_group = &$groups->getGroup($oldGroupName);
            $new_group = &$groups->getGroup($newGroupName);
            $old_group->removeUser($userID);
            $new_group->addUser($userID);
            return true;
        }
        return false;
    }

    /**
     * Function that sets the user as a wanting user for the given project
     * It checks the user has no other relationship with that project
     *
     * @param string $userID The user identifier
     * @return boolean True if the user could be set to wanting
     */
     function addUserRequest($userID)
     {
        $groups = &Group::singleton();

        if (!$groups->userIsInGroup($userID, $this->_top_group_name, true)) {
            $the_group = &$groups->getGroup($this->_default_groups[USERS_WANTING]);
            $the_group->addUser($userID);
            return true;
        }
        return false;
     }

    /**
     * Function that sets the user as an accepted user for the given project
     * It must check whether the user is currently requesting to join
     * Depending on how the Source Control accounts creation is handled
     * the user should be added to the developers group or to the accepted group
     *
     * @param string $userID The user identifier
     * @return boolean True if the user could be set to accepted
     */
    function acceptUserRequest($userID)
    {
        $groups = &Group::singleton();

        $accepted_name = $this->_default_groups[USERS_ACCEPTED];
        $wantings_name = $this->_default_groups[USERS_WANTING];
        $new_groupID = $groups->getGroupId($accepted_name);
        $old_groupID = $groups->getGroupId($wantings_name);
        return $this->changeUserGroup($userID, $old_groupID, $new_groupID);
    }

     /**
     * Function that refuses a user request to join the given project
     * It must check whether the user is currently a wanting
     *
     * @param string $userID The user identifier
     * @return boolean True if the user could be set to refused
     */
    function refuseUserRequest($userID)
    {
        $groups = &Group::singleton();

        $refused_name = $this->_default_groups[USERS_REFUSED];
        $wantings_name = $this->_default_groups[USERS_WANTING];
        $new_groupID = $groups->getGroupId($refused_name);
        $old_groupID = $groups->getGroupId($wantings_name);
        return $this->changeUserGroup($userID, $old_groupID, $new_groupID);
    }

    /**
     * This function creates a shared task list in Nag
     * and sets the correct permissions for it:
     *     Every Developers shared tasks list can be accessed by Administrators and
     *     the appropriate Developers subgroup.
     *     For every Administrators shared tasks list only the appropriate subgroup
     *     has access to it.
     *
     * @param string $groupName The group we want to add the tasks list for.
     * @return mixed True if all went ok. PEAR_Error if there was a problem.
     */
    function createSharedList($groupName)
    {
        global $registry;

        $apiargs = array();
        $apiargs['name'] = $groupName;
        $apiargs['description'] = sprintf(_("Shared tasks list for the Group '%s' "), $groupName);
        // Create the list
        $listId = $registry->call('tasks/addTasklist', $apiargs);

        // Set the correct permissions for it.
        if (is_a($listId, 'PEAR_Error')) {
            return $listId;
        }

        require_once 'Horde/Share.php';

        $shares = &Horde_Share::singleton('nag');
        $share = &$shares->getShare($listId);
        if (is_a($share, 'PEAR_Error')) {
            return $share;
        }

        $permission = &$share->getPermission();
        if (is_a($permission, 'PEAR_Error')) {
            return $permission;
        }

        $groups = &Group::singleton();
        $group = &$groups->getGroup($groupName);
        $theGroupID = $group->getId();
        $adminsGroup = $groups->getGroup($this->_default_groups[USERS_ADMINS]);
        $adminsId = $adminsGroup->getId();
        // Show, Read and Edit for the group
        $permission->addGroupPermission($theGroupID, PERMS_SHOW, false);
        $permission->addGroupPermission($theGroupID, PERMS_READ, false);
        $permission->addGroupPermission($theGroupID, PERMS_EDIT, false);

        // Add admins to the permissions
        $permission->addGroupPermission($adminsId, PERMS_SHOW, false);
        $permission->addGroupPermission($adminsId, PERMS_READ, false);
        $permission->addGroupPermission($adminsId, PERMS_EDIT, false);

        $share->setPermission($permission);
        $share->save();
        return true;
    }

    /**
     * TODO
     */
    function deleteSharedList($groupName)
    {
    }

    /**
     * Creates the default users groups.
     * Directly sets the owner as an administrator for that project.
     * @return mixed                 The top level group id if all went ok or PEAR_Error
     *                                            if there was a problem
     */
    function createDefaultGroups()
    {
        $groups = &Group::singleton();

        // Create the top level group and try to add it.
        if ($groups->exists($this->_top_group_name)) {
            $top_level = &$groups->getGroup($this->_top_group_name);
        } else {
            $top_level = &$groups->newGroup($this->_top_group_name);
            $result = $groups->addGroup($top_level);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        // Try to add the groups.
        foreach ($this->_default_groups as $id => $group_name) {
            $new_group = &$groups->newGroup($group_name);
            if ($id == USERS_ADMINS) {
                // Add the owner to the admins group.
                $new_group->addUser($this->data['project_coordinator_id'], false);
            }
            $result = $groups->addGroup($new_group);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }
        return $top_level->getID();
    }

    /**
     * Removes the Project groups.
     *
     * @return mixed  Category Id of all went ok or PEAR_Error if there was a problem
     */
    function deleteGroups()
    {
        $groups = &Group::singleton();
        $top_level = &$groups->getGroup($this->_top_group_name);
        $result = $groups->removeGroup($top_level,true);
        return $result;
    }

     /**
     * This function creates the Projects Task Lists for the given project
     * This allows the developers and administrators to share tasks
     * Two lists are created one shared between administrators
     * and the other shared between the developers (this includes administrators)
     * The function creates the default Task lists on project creation.
     * If there is any problem the function takes care of reverting changes back
     *
     * @return mixed              True if all went ok. PEAR_Error if there was any problem
     */
    function createDefaultTaskLists()
    {
        $result = $this->createSharedList($this->_default_groups[USERS_DEVS]);
        if (is_a($result, 'PEAR_Error')) {
            // Return the error
            return $result;
        }
        $result = $this->createSharedList($this->_default_groups[USERS_ADMINS]);
        return $result;
    }

     /**
     * TODO: Browse all the sub-groups of devs and admins and remove the shared lists
     *
     * This function deletes the Projects Task Lists for the given project
     * This function would be called on project deletion
     */
    function deleteTaskLists()
    {
        $this->deleteSharedList($this->_default_groups[USERS_ADMINS]);
        $this->deleteSharedList($this->_default_groups[USERS_DEVS]);
    }

    /**
     * This function shows the groups tree for a given project
     * If the user is a project administrator it shows also links
     * for team management
     *
     * @param string $current     This is the currently selected node. Allows to keep track of
     *                                              user's interactions. Default = no current node
     */
    function showProjectGroupsTree($current = null)
    {
        global $registry;

        require_once 'Horde/Tree.php';

        $projectData = $this->toArray();

        /* Get the project tree. */
        $groups = &Group::singleton();
        $nodes = $groups->_datatree->get(DATATREE_FORMAT_FLAT, $this->_top_group_name, true, $projectData['project_name'], -1, true);

        $spacer = '&nbsp;&nbsp;&nbsp;&nbsp;';
        $graph_horde = $registry->getImageDir('horde');
        $icondir = array('icondir' => $graph_horde);
        $group_node = $icondir + array('icon' => 'group.png');
        $base_url = Horde::applicationUrl('team.php?project_id=' . $projectData['project_id']);
        $edit_img = Horde::img('edit.png', _("Edit Group"),'hspace="2"', $registry->getImageDir('horde'));


        /* Set up the tree. */
        $tree = &Horde_Tree::singleton('team_tree', 'javascript');
        $tree->setOption(array('border' => '0', 'class' => 'item', 'cellpadding' => '0', 'cellspacing' => '0', 'alternate' => true));

        $current_parents = $groups->_datatree->getParentList($current);

        foreach ($nodes as $cid => $node) {
            $node_class = ($current == $cid) ? array('class' => 'selected') : array();
            if ($node == $projectData['project_name']) {
                $link = Horde::link($base_url, _("Show all users")) . $edit_img . '</a>';
                $base_node_params = $icondir + array('icon' => 'administration.png');
                $tree->addNode($cid, null, $projectData['project_name'], 0, true, $base_node_params + $node_class, $link);
            } else {
                $group_extra = array();
                $edit_link = Horde::link(Util::addParameter($base_url, 'groupID', $cid), _("Edit Group")) . $edit_img . '</a>';
                array_push($group_extra, $edit_link);
                $parent_id = $groups->_datatree->getParent($node);
                $tree->addNode($cid, $parent_id, DataTree::getShortName($node), substr_count($node, ':') ,(isset($current_parents[$cid])) ? true : false, $group_node + $node_class, $group_extra);
            }
        }
        echo "<br />";
        $tree->renderTree();
    }

    /**
     * Shows the Project information
     * Every link is maintained using the same base url
     *
     * @param string optional $insertBreak Insert a break before adding the content?
     * @param boolean optional $short Show all the information if false.
     *                                                 Use a summarised template if True.
     */
    function showInformation($insertBreak = false, $short = false)
    {
        echo sprintf('The class %s should implement the method showInformation', get_class($this));
    }

    function _makeUsersTable($users, $listName, $sortby, $sortdir, $showActions = false, $insertBreak = true)
    {
        global $registry, $thor, $browser;

        // Show the Join Link if appropriate
        if (!$this->isUserOfKind(Auth::getAuth(), USERS_TEAM) &&
            !$this->isUserOfKind(Auth::getAuth(), USERS_WANTING) &&
            $this->get('project_status') == THOR_STATUS_ACTIVE) {

            $joinUrl = Horde::applicationUrl(Util::addParameter('view.php',
                array('project_id' => $thor->_projectsContainer->getProjectId($this->getName()),
                            'sortby' => $sortby,
                            'sortdir' => $sortdir,
                            'join' => '1')));

            $rightHeaderText = Horde::link($joinUrl , _("Join the project")) . _("Join the project") . '</a>';
        } else {
            $rightHeaderText = '';
        }

        $num_users = count($users);
        $leftHeaderText = $num_users == 1 ?
            sprintf(_("1 User in '%s'"), $listName) :
            sprintf(_("%d Users in '%s'"), $num_users, $listName);
        require THOR_TEMPLATES . '/section-header.inc';
        if (!$num_users) {
            require THOR_TEMPLATES . '/team/empty.inc';
        } else {
            require THOR_TEMPLATES . '/team/header_team.inc';
            foreach ($users as $userID) {
                if ($this->isUserOfKind($userID, USERS_ADMINS)) {
                    $role = Horde::img('myaccount.png', _("Administrator"), 'hspace="2"', $registry->getImageDir('horde'));
                } elseif ($this-> isUserOfKind($userID, USERS_DEVS)) {
                    $role = Horde::img('user.png', _("Developer"), 'hspace="2"', $registry->getImageDir('horde'));
                } else {
                    $role = _("Other");
                }
                require THOR_TEMPLATES . '/team/team.inc';
            }
            require THOR_TEMPLATES . '/team/footer.inc';
        }
        require THOR_TEMPLATES . '/team/footer.inc';
    }

    /**
     * Shows the Users of the given group.
     * @param mixed $group The DataTreeObject_Group or the group id
     */
    function showUsers(&$group, $sortby, $sortdir, $subGroups = true, $showActions = false, $insertBreak = true)
    {
        if (!isset($group)) {
            return;
        } else {
            if (!is_a($group, 'DataTreeObject_Group')) {
                $groups = &Group::singleton();
                $group = &$groups->getGroupById($group);
            }
        }

        $members = $subGroups ? $group->listAllUsers() : $group->listUsers();
        $group_name = $group->getName();
        $this->_makeUsersTable($members, $group_name, $sortby, $sortdir, $showActions, $insertBreak);
    }

    function showTeam($sortby, $sortdir, $showActions = false, $insertBreak = true)
    {
        $users = $this->listUsers(USERS_TEAM, $sortby, $sortdir);
        $this->_makeUsersTable($users, _("Project Team"), $sortby, $sortdir, $showActions, $insertBreak);
    }

    function showSubProjects($projects_sortby, $projects_sortdir, $insertBreak = true)
    {
        global $thor, $registry, $prefs;

        $criteria = array('AND' => array(
            array('field' => 'name', 'op'  => '=', 'test'  => 'project'),
            array('field' => 'key', 'op' => '=', 'test'  => 'status'),
            array('field' => 'value', 'op' => '=', 'test'  => THOR_STATUS_ACTIVE)));

        $subProjects = &$thor->_projectsContainer->getProjectsByCriteria($criteria, $this->getName(), true, true);

        $sortedList = $thor->sortObjects($subProjects, $projects_sortby, $projects_sortdir);

        $num_projects = count($sortedList);
        $leftHeaderText = $num_projects == 1 ? _("1 Subproject") : sprintf(_("%d Subprojects"), $num_projects);
        $rightHeaderText = '';

        require THOR_TEMPLATES . '/section-header.inc';

        if ($num_projects) {
            require THOR_TEMPLATES . '/list/project_headers.inc';
            // List the projects.
            foreach ($sortedList as $project) {
                require THOR_TEMPLATES . '/list/project_summaries.inc';
            }
        } else {
            require THOR_TEMPLATES . '/list/empty.inc';
        }
        require THOR_TEMPLATES . '/list/footer.inc';
    }

    function showItems($items_sortby, $items_sortdir, $insertBreak = false)
    {
        global $thor, $registry, $prefs;
        // Get the items
        $itemsList = &$thor->_itemsContainer->getItems($this->getName(), DATATREE_ROOT, true, true);
        $sortedList = $thor->sortObjects($itemsList, $items_sortby, $items_sortdir);

        $num_items = count($itemsList);

        $leftHeaderText = $num_items == 1 ? _("1 Project Item") : sprintf(_("%d Project Items"), $num_items);
        $rightHeaderText = '';

        require THOR_TEMPLATES . '/section-header.inc';
        if ($num_items) {
            require THOR_TEMPLATES . '/list/item_headers.inc';
            // List the items.
            foreach ($sortedList as $itemID => $item) {
                require THOR_TEMPLATES . '/list/item_summaries.inc';
            }
            require THOR_TEMPLATES . '/list/footer.inc';
        } else {
            require THOR_TEMPLATES . '/list/empty.inc';
        }
        require THOR_TEMPLATES . '/list/footer.inc';
    }

    /**
     * Download the Project
     * Should be implemented by any subclass
     *@return mixed True if data could be sent. PEAR_Error otherwise.
     *
     */
    function download()
    {
        return raise::PEAR_Error(sprintf('The class %s must implement the method showInformation', get_class($this)));
    }

    /**
     * Do Project initialization.
     * This function takes care of updating the backend
     * Generates:
     *      - default Groups of Users
     *      - shared Tasks lists
     *
     * TODO: Check Notification handling
     *
     * @return mixed True if all went ok. False if there was any problem.
     */
    function initialize()
    {
       global $notification, $thor;
       // Create the Project Groups
       $group_id = $this->createDefaultGroups();
       if (is_a($group_id, 'PEAR_Error')) {
           $notification->push(sprintf(_("Unable to create project groups: %s"), $group_id->getMessage()), 'horde.error');
           $this->deleteGroups();
           return $group_id;
       }

       // Create the Project Tasks Lists
       $result_tasks = $this->createDefaultTaskLists();
       if (is_a($result_tasks, 'PEAR_Error')) {
           $notification->push(sprintf(_("Could not create default task lists: %s"), $result_tasks->getMessage()), 'horde.error');
           $this->deleteGroups();
           $this->deleteTaskLists();
           return $result_tasks;
       }

       $this->data['project_status'] = THOR_STATUS_ACTIVE;
       return $thor->_projectsContainer->saveProject($this);
    }

    /**
     * Write protect the Project.
     * This can be used by Project administrators to write protect projects
     *
     *@param boolean $value True -> Write protect it.
     *@return mixed PEAR_Error is something went wrong. Id of the project otherwise
     */
    function setWriteProtected($value)
    {
        global $thor;

        $this->data['project_write_protected'] = $value ? 1 : 0;
        $result = $thor->_projectsContainer->saveProject($this);
        return $result;
    }

    /**
     * Destroys the Project
     * Deletes Groups, Task Lists and the Items
     */
    function _destroy()
    {
        global $thor;

        if ($this->data['project_status'] != THOR_STATUS_REQUEST) {
            $this->deleteGroups();
            $this->deleteTaskLists();
        }

        //Foreach sub project delete it also
        $subProjects = &$thor->_projectsContainer->getProjects(THOR_STATUS_ANY, $this->getName(), 1, true, false);
        $projectIds = array_keys($subProjects);
        foreach ($projectIds as $projectId) {
            $thor->_projectsContainer->deleteProject($projectId);
        }

        //For each Project Item -> destroy it
        $itemsList = &$thor->_itemsContainer->getItems($this->getName(), 1, true, false);
        $itemIds = array_keys($itemsList);
        foreach ($itemIds as $id) {
            $thor->_itemsContainer->deleteItem($id);
        }

        // Remove the top level
        $thor->_itemsContainer->_items->remove($this->getName());
    }

}

/**
 * ProjectForm Class
 *
 * $Horde: thor/lib/Project.php,v 1.41 2004/12/20 19:14:53 chuck Exp $
 *
 * @author  Carlos Pedrinaci <cpedrinaci@yahoo.es>
 * @version $Revision: 1.41 $
 * @package Thor
 */
class ProjectForm extends Horde_Form {

    var $_useFormToken = false;

    function ProjectForm(&$vars)
    {
        global $thor;

        $pid = $vars->get('project_id');
        $project_parent_id = $vars->get('project_parent_id');
        $title = empty($pid) ? _("Create Project - Step 1") : _("Edit Project");

        parent::Horde_Form($vars, $title);

        $this->addHidden('', 'project_id', 'int', false);
        $this->addVariable(_("Project Name"), 'project_name', 'text', true);
        $kinds_list = $thor->getProjectNames();
        // Don't allow to change the Project kind when editting
        $this->addVariable(_("Project Kind"), 'project_kind', 'enum', true, empty($pid) ? false : true, null, array($kinds_list));
        $projects_list = $thor->formatDataTree($thor->_projectsContainer->listUserProjects(Auth::getAuth(), THOR_STATUS_EDITABLE, USERS_ADMINS));
        $this->addVariable(_("Parent Project"), 'project_parent_id', 'enum', false, false, null, array($projects_list));
        $this->addVariable(_("Project Description"), 'project_description', 'longtext', true, false, null);
    }

}
